package com.zhilian.resolver.resolverReport;
import com.zhilian.common.core.utils.SpringUtils;
import com.zhilian.common.resolver.domain.ResolverReportData;
import com.zhilian.resolver.service.ResolverEventService;
import com.zhilian.resolver.service.ResolverReportInfoService;
import com.zhilian.resolver.service.impl.eventMalfunction.MalfunctionEventServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.zhilian.common.resolver.constants.MqttConnectConstants.BROKER_URL;
import static com.zhilian.common.resolver.constants.MqttConnectConstants.CLIENT_ID;

/**
 * @ClassName ResolverMqttMsg
 * @Description 获取mqtt报文消息 解析ResolverReportData vehicleData
 * @Author Can.J
 * @Date  2024/3/30 8:36
 */
@Slf4j
@Component
public class ResolverMqttMsg {
    @Autowired
    private MalfunctionEventServiceImpl malfunctionEventService;

    @Autowired
    private ResolverReportInfoService resolverReportInfoService;

    @Autowired
    public ResolverMqttMsg(ResolverReportInfoService resolverReportInfoService) {
        this.resolverReportInfoService = resolverReportInfoService;
    }

    public static ArrayList<String> stringEvents = new ArrayList<>(){
        {
            add("malfunction");     //故障事件
            add("event-realTimeTrajectory"); //实时轨迹数事件
            add("event-storage");      //存储服务事件
            add("geofence");        //电子围栏事件
        }
    };
//    @Scheduled(fixedRate = 500) //执行频率500毫秒
    // 执行频率5秒
    @Scheduled(cron = "0/5 * * * * ?")
    public void start() {

        try {
            // 创建客户端
            MqttClient mqttClient = new MqttClient(BROKER_URL, CLIENT_ID);

            // 设置连接参数
            MqttConnectOptions options = new MqttConnectOptions();

            // 设置是否清除会话
            options.setCleanSession(true);
            // 连接
            mqttClient.connect(options);

            // 设置回调
            mqttClient.setCallback(new MqttCallback() {

                @Override
                public void connectionLost(Throwable throwable) {
                    log.error("Mqtt[{}-{}]连接断开:[{}]", CLIENT_ID, BROKER_URL, throwable.getMessage(), throwable);
                }

                @Override
                public void messageArrived(String topic, MqttMessage message) throws Exception {
                    log.info("主题='{}':消息内容={}", topic, new String(message.getPayload()));

                    // 解析后的字符串
                    // { ~2BL9IQ15JX3IEL4OZ1712199325613116.659444039.521254072.00030.4900000034800080000163190000D303011.00360000929212560134000281007032000062102.93000070000030120007490003000400016000090000084000011111111111111111z~ }
                    String str = hexStringToString(new String(message.getPayload()));

                    List<ResolverReportData> vehicleDataList = parseVehicleData(str);

                    for (ResolverReportData vehicleData : vehicleDataList) {
                        //str-->vehicleDataList-->vehicleData实例
                        log.info("解析到车辆数据：{}", vehicleData);

                        for (String stringEvent : stringEvents) {
                            ResolverEventService resolverEventService =SpringUtils.getBean(stringEvent);
                            resolverEventService.execute(vehicleData);

                        }
                    }
                    log.info("字符串：{}",str);
                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken token) {
                    log.info("消息已成功投递：{}",token);
                }
            });

            mqttClient.subscribe("cartest",0);

            Thread.sleep(1000*60*10);

//            mqttClient.disconnect();
        } catch (MqttException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }

    }
    /**
     * 16进制转换成为string类型字符串
     *
     * @param s
     * @return
     */
    public static String hexStringToString(String s) {
        if (s == null || s.equals("")) {
            return null;
        }
        s = s.replace(" ", "");
        byte[] baKeyword = new byte[s.length() / 2];
        for (int i = 0; i < baKeyword.length; i++) {
            try {
                baKeyword[i] = (byte) (0xff & Integer.parseInt(s.substring(i * 2, i * 2 + 2), 16));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        try {
            s = new String(baKeyword, StandardCharsets.UTF_8);
        } catch (Exception e1) {
            e1.printStackTrace();
            return s;
        }
        return s;
    }

    // 解析数据 字符串切割 获取适合的数据
    private static List<ResolverReportData> parseVehicleData(String str) {
        List<ResolverReportData> vehicleDataList = new ArrayList<>();
        ResolverReportData vehicleData = new ResolverReportData();

        vehicleData.setVin(str.substring(1,18));

        log.info("vin=="+vehicleData.getVin());

        //时间
        String tim =str.substring(18,31);
        long timestamp = Long.parseLong(tim);

        Date date = new Date(timestamp);
        vehicleData.setCreateTime(date);

        //经度
        String lt = str.substring(31,42);
        // 如果末尾是零，则舍去
        int endIndex = lt.length() - 1;
        while (lt.charAt(endIndex) == '0'){
            endIndex--;
        }

        String longitude = lt.substring(0, endIndex + 1);
        vehicleData.setLongitude(longitude);

        //维度
        String latitudeIndex =str.substring(42,52);
        int endIndexT = latitudeIndex.length() - 1;
        while (latitudeIndex.charAt(endIndexT) == '0'){
            endIndexT--;
        }

        String latitude = latitudeIndex.substring(0, endIndexT + 1);
        vehicleData.setLatitude(latitude);

        //速度speed
        String speed =str.substring(52,58);
        vehicleData.setSpeed(speed);

        //里程
        BigDecimal mileage= new BigDecimal(str.substring(58,69));
        mileage=mileage.stripTrailingZeros();
        vehicleData.setMileage(mileage);

        //总电压
        String voltage =str.substring(69,75);
        while (voltage.endsWith("0")) {
            voltage = voltage.substring(0, voltage.length() - 1); // 去除末尾的零
        }
        vehicleData.setVoltage(voltage);

        //总电流
        String current =str.substring(75,80);
        while (current.endsWith("0")){
            current=current.substring(0,current.length()-1);
        }
        vehicleData.setCurrent(current);

        //绝缘电阻 resistance
        String res =str.substring(80,89);
        String resistance = res.substring(0, 5);
        vehicleData.setResistance(resistance);

        //档位
        String gear =str.substring(89,90);
        vehicleData.setGear(gear);

        //accelerationPedal 加速踏板行程值
        String accelerationPedal =str.substring(90,91);
        vehicleData.setAccelerationPedal(accelerationPedal);

        //brakePedal 制动踏板行程值
        String brakePedal =str.substring(92,93);
        vehicleData.setBrakePedal(brakePedal);

        //fuelConsumptionRate  燃料消耗率
        String fuelConsumptionRate =str.substring(94,99);
        vehicleData.setFuelConsumptionRate(fuelConsumptionRate);

        //motorControllerTemperature 电机控制器温度
        String motorControllerTemperature =str.substring(99,105);
        while (motorControllerTemperature.endsWith("0")){
            motorControllerTemperature=motorControllerTemperature.substring(0,motorControllerTemperature.length()-1);
        }
        vehicleData.setMotorControllerTemperature(motorControllerTemperature);

        //motorSpeed 电机转速
        String motorSpeed =str.substring(105,110);
        vehicleData.setMotorSpeed(motorSpeed);

        //motorTorque 电机转矩
        String motorTorque =str.substring(110,114);
        while (motorTorque.endsWith("0")){
            motorTorque=motorTorque.substring(0,motorTorque.length()-1);
        }
        vehicleData.setMotorTorque(motorTorque);


        //motorTemperature 电机温度
        String motorTemperature =str.substring(114,120);
        while (motorTemperature.endsWith("0")){
            motorTemperature=motorTemperature.substring(0,motorTemperature.length()-1);
        }
        vehicleData.setMotorTemperature(motorTemperature);

        //motorVoltage 电机电压
        String motorVoltage =str.substring(120,125);
        while (motorVoltage.endsWith("0")){
            motorVoltage=motorVoltage.substring(0,motorVoltage.length()-1);
        }
        vehicleData.setMotorVoltage(motorVoltage);

        //motorCurrent 电机电流
        String motorCurrent =str.substring(125,133);
        while (motorCurrent.endsWith("0")){
            motorCurrent=motorCurrent.substring(0,motorCurrent.length()-1);
        }
        vehicleData.setMotorCurrent(motorCurrent);

        //remainingBattery 动力电池剩余电量SOC
        BigDecimal remainingBattery = new BigDecimal(str.substring(133,138));
        vehicleData.setRemainingBattery(remainingBattery);

        //maximumFeedbackPower 当前状态允许的最大反馈功率
        String maximumFeedbackPower =str.substring(139,144);
        while (maximumFeedbackPower.endsWith("0")){
            maximumFeedbackPower=maximumFeedbackPower.substring(0,maximumFeedbackPower.length()-1);
        }
        vehicleData.setMaximumFeedbackPower(maximumFeedbackPower);

        //maximumDischargePower 当前状态允许最大放电功率
        String maximumDischargePower =str.substring(145,151);
        while (maximumDischargePower.endsWith("0")){
            maximumDischargePower=maximumDischargePower.substring(0,maximumDischargePower.length()-1);
        }
        vehicleData.setMaximumDischargePower(maximumDischargePower);

        //selfCheckCounter BMS自检计数器
        String selfCheckCounter =str.substring(151,153);
        String selfCheckCounterReplace = selfCheckCounter.replace("0", "");
        vehicleData.setSelfCheckCounter(selfCheckCounterReplace);

        //totalBatteryCurrent 动力电池充放电电流
        String totalBatteryCurrent =str.substring(153,158);
        while (totalBatteryCurrent.endsWith("0")){
            totalBatteryCurrent=totalBatteryCurrent.substring(0,totalBatteryCurrent.length()-1);
        }
        vehicleData.setTotalBatteryCurrent(totalBatteryCurrent);

        //totalBatteryVoltage 动力电池负载端总电压V3
        String totalBatteryVoltage =str.substring(158,164);
        while (totalBatteryVoltage.endsWith("0")){
            totalBatteryVoltage=totalBatteryVoltage.substring(0,totalBatteryVoltage.length()-1);
        }
        vehicleData.setTotalBatteryVoltage(totalBatteryVoltage);

        //singleBatteryMaxVoltage 单次最大电压
        String singleBatteryMaxVoltage =str.substring(164,168);
        while (singleBatteryMaxVoltage.endsWith("0")){
            singleBatteryMaxVoltage=singleBatteryMaxVoltage.substring(0,singleBatteryMaxVoltage.length()-1);
        }
        vehicleData.setSingleBatteryMaxVoltage(singleBatteryMaxVoltage);

        //singleBatteryMinVoltage 单体电池最低电压
        String singleBatteryMinVoltage =str.substring(168,172);
        while (singleBatteryMinVoltage.endsWith("0")){
            singleBatteryMinVoltage=singleBatteryMinVoltage.substring(0,singleBatteryMinVoltage.length()-1);
        }

        vehicleData.setSingleBatteryMinVoltage(singleBatteryMinVoltage);

        //singleBatteryMaxTemperature 单体电池最高温度
        String singleBatteryMaxTemperature =str.substring(172,178);
        while (singleBatteryMaxTemperature.endsWith("0")){
            singleBatteryMaxTemperature=singleBatteryMaxTemperature.substring(0,singleBatteryMaxTemperature.length()-1);
        }
        vehicleData.setSingleBatteryMaxTemperature(singleBatteryMaxTemperature);

        //singleBatteryMinTemperature 单体电池最低温度
        String singleBatteryMinTemperature =str.substring(178,184);
        while (singleBatteryMinTemperature.endsWith("0")){
            singleBatteryMinTemperature=singleBatteryMinTemperature.substring(0,singleBatteryMinTemperature.length()-1);
        }
        vehicleData.setSingleBatteryMinTemperature(singleBatteryMinTemperature);

        //availableBatteryCapacity 可用电池容量
        String availableBatteryCapacity =str.substring(184,190);
        while (availableBatteryCapacity.endsWith("0")){
            availableBatteryCapacity=availableBatteryCapacity.substring(0,availableBatteryCapacity.length()-1);
        }
        vehicleData.setAvailableBatteryCapacity(availableBatteryCapacity);

        //vehicleStatus 车辆状态
        int vehicleStatus = Integer.parseInt(str.substring(190,191));
        vehicleData.setVehicleStatus(vehicleStatus);

        //chargingStatus 充电状态
        int chargingStatus = Integer.parseInt(str.substring(191,192));
        vehicleData.setChargingStatus(chargingStatus);

        //operatingStatus 运行状态
        int operatingStatus = Integer.parseInt(str.substring(192,193));
        vehicleData.setOperatingStatus(operatingStatus);

        //socStatus SOC
        int socStatus = Integer.parseInt(str.substring(193,194));
        vehicleData.setSocStatus(socStatus);

        //chargingEnergyStorageStatus 可充电储能装置工作状态
        int chargingEnergyStorageStatus = Integer.parseInt(str.substring(194,195));
        vehicleData.setChargingEnergyStorageStatus(chargingEnergyStorageStatus);

        //driveMotorStatus 驱动电机状态
        int driveMotorStatus = Integer.parseInt(str.substring(195,196));
        vehicleData.setDriveMotorStatus(driveMotorStatus);

        //positionStatus 定位是否有效
        int positionStatus = Integer.parseInt(str.substring(196,197));
        vehicleData.setPositionStatus(positionStatus);

        //easStatus EAS(汽车防盗系统)状态
        int easStatus = Integer.parseInt(str.substring(197,198));
        vehicleData.setEasStatus(easStatus);

        //ptcStatus PTC(电动加热器)状态
        int ptcStatus = Integer.parseInt(str.substring(198,199));
        vehicleData.setPtcStatus(ptcStatus);

        //epsStatus
        int epsStatus = Integer.parseInt(str.substring(199,200));
        vehicleData.setEpsStatus(epsStatus);

        //absStatus EPS(电动助力系统)状态
        int absStatus = Integer.parseInt(str.substring(200,201));
        vehicleData.setAbsStatus(absStatus);

        //mcuStatus MCU(电机/逆变器)状态
        int mcuStatus = Integer.parseInt(str.substring(201,202));
        vehicleData.setMcuStatus(mcuStatus);

        //heatingStatus 动力电池加热状态
        int heatingStatus = Integer.parseInt(str.substring(202,203));
        vehicleData.setHeatingStatus(heatingStatus);

        //batteryStatus 动力电池当前状态
        int batteryStatus = Integer.parseInt(str.substring(203,204));
        vehicleData.setBatteryStatus(batteryStatus);

        //batteryInsulationStatus 动力电池保温状态
        int batteryInsulationStatus = Integer.parseInt(str.substring(204,205));
        vehicleData.setBatteryInsulationStatus(batteryInsulationStatus);

        //dcdcStatus DCDC(电力交换系统)状态
        int dcdcStatus = Integer.parseInt(str.substring(205,206));
        vehicleData.setDcdcStatus(dcdcStatus);

        //chgStatus CHG(充电机)状态
        int chgStatus = Integer.parseInt(str.substring(206,207));
        vehicleData.setChgStatus(chgStatus);

        vehicleDataList.add(vehicleData);

        return vehicleDataList;
    }
}
