package com.zhilian.resolver.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhilian.common.core.utils.StringUtils;
import com.zhilian.common.resolver.domain.ResolverReportData;
import com.zhilian.common.resolver.domain.ResolverReportInfo;
import com.zhilian.resolver.mapper.ResolverReportInfoMapper;
import com.zhilian.resolver.service.ResolverReportInfoService;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.bulk.BulkItemResponse;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.xcontent.XContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.time.Duration;
import java.util.List;
import java.util.Objects;

/**
  * @ClassName ReportServiceImpl
  * @Description 报文业务实现层
  * @Author Can.J
  * @Date  2024/3/26 22:31
 */
@Slf4j
@Service
public class ResolverReportInfoServiceImpl extends ServiceImpl<ResolverReportInfoMapper, ResolverReportInfo> implements ResolverReportInfoService {
    @Resource
    private RedisTemplate<String,Object> redisTemplate;

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    @Autowired
    private ResolverReportInfoMapper reportMapper;

    /**
     *  查询报文数据
     * @param report
     * @return
     */
    @Override
    public List<ResolverReportInfo> pageQuery(ResolverReportInfo report) {
        List<ResolverReportInfo> resultList;
        //redis中查询
        log.info("从redis查询数据");
        String cacheKey=generateCacheKey(report);
        resultList =(List<ResolverReportInfo>) redisTemplate.opsForValue().get(cacheKey);
        if(resultList==null){
            //如果redis中不存在，则从数据库中查询
            log.info("从数据库查询数据");
            LambdaQueryWrapper<ResolverReportInfo> queryWrapper = new LambdaQueryWrapper<>();
            if (StringUtils.isNotEmpty(report.getVin())){
                queryWrapper.like(ResolverReportInfo::getVin,report.getVin());
            }
            //status int类型 精确查询
            if(Objects.nonNull(report.getStatus())){
                queryWrapper.eq(ResolverReportInfo::getStatus,report.getStatus());
            }
            queryWrapper.eq(ResolverReportInfo::getIsDelete, 1); // 添加 isDelete == 1 的条件
            resultList =this.list(queryWrapper);

            // 将从数据库中获取的数据存入 Redis，设置过期时间10分钟
            redisTemplate.opsForValue().set(cacheKey,resultList, Duration.ofMinutes(10));

            //查询结果从数据库获取之后，将其批量写入Elasticsearch
            try {
                bulkInsert(resultList);
            } catch (IOException e) {
                log.info("报文数据同步ES错误",e.getMessage());
            }
        }
        return resultList;
    }


    /**
     *  存储事件
     * @param vehicleData
     */
    @Override
    public void saveDataToDatabase(ResolverReportData vehicleData) {
        reportMapper.reportMapper(vehicleData);
    }

    /**
     *  故障事件
     * @param resolverReportData
     */
    @Override
    public void saveDataToDatabaseByMalfunction(ResolverReportData resolverReportData) {
         reportMapper.saveDataToDatabaseByMalfunction(resolverReportData);
    }


    /**
     * 生成缓存的key
     * @param report
     * @return
     */
    private String generateCacheKey(ResolverReportInfo report) {
        return "report_"+report.getVin();
    }

    public void bulkInsert(List<ResolverReportInfo> reports) throws IOException {
        BulkRequest bulkRequest = new BulkRequest();
        reports.forEach(report -> {
            // 创建一个索引请求，指定索引名称和文档ID
            IndexRequest indexRequest = new IndexRequest("report");
            indexRequest.id(report.getReportId().toString());
            // 将对象转换为JSON字符串
            String jsonString = JSON.toJSONString(report);
            // 将JSON字符串设置为索引请求的文档内容
            indexRequest.source(jsonString, XContentType.JSON);
            // 将索引请求添加到批量请求中
            bulkRequest.add(indexRequest);
        });
        BulkResponse bulkResponse = restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
        if(bulkResponse.hasFailures()){
            //处理失败的情况
            for (BulkItemResponse item : bulkResponse.getItems()) {
                if(item.isFailed()){
                    log.error("报文数据同步到ES失败:{}",item.getFailureMessage());
                }
            }
        }else{
            log.info("报文同步ES成功：{}条",reports.size());
        }
    }
}
